/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.jcr.webdav;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.commons.logging.Log;
import org.apache.jackrabbit.server.AbstractWebdavServlet;
import org.apache.jackrabbit.webdav.DavLocatorFactory;
import org.apache.jackrabbit.webdav.DavResource;
import org.apache.jackrabbit.webdav.DavResourceFactory;
import org.apache.jackrabbit.webdav.DavSessionProvider;
import org.apache.jackrabbit.webdav.WebdavRequest;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.WebdavService;
/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */

public class SimpleJCRWebdavServlet extends AbstractWebdavServlet {
  
  private static Log log = ExoLogger.getLogger("jcr:SimpleWebdavServlet");

  public static final String INIT_PARAM_RESOURCE_PATH_PREFIX = "resource-path-prefix";
  public static final String INIT_PARAM_AUTHENTICATE_HEADER = "authenticate-header";
  public static final String CTX_ATTR_RESOURCE_PATH_PREFIX = "jackrabbit.webdav.simple.resourcepath";
 
  private String resourcePathPrefix;

  private WebdavService davService;
  
  private String authenticateHeader;
  
  @Override
  public void init() throws ServletException {
    
    ExoContainer container = ExoContainerContext.getCurrentContainer();
    if(container == null)
      throw new ServletException("Current ExoContainer is not set");
    davService = (WebdavService)container.getComponentInstanceOfType(WebdavService.class);
    log.info("WebDavService = " + davService);
    

    resourcePathPrefix = getInitParameter(INIT_PARAM_RESOURCE_PATH_PREFIX);
    if (resourcePathPrefix == null) {
        log.debug("Missing path prefix > setting to empty string.");
        resourcePathPrefix = "";
    } else if (resourcePathPrefix.endsWith("/")) {
        log.debug("Path prefix ends with '/' > removing trailing slash.");
        resourcePathPrefix = resourcePathPrefix.substring(0, resourcePathPrefix.length() - 1);
    }
    getServletContext().setAttribute(CTX_ATTR_RESOURCE_PATH_PREFIX, resourcePathPrefix);
    log.info(INIT_PARAM_RESOURCE_PATH_PREFIX + " = '" + resourcePathPrefix + "'");

    authenticateHeader = getInitParameter(INIT_PARAM_AUTHENTICATE_HEADER);
    if (authenticateHeader == null) {
        authenticateHeader = DEFAULT_AUTHENTICATE_HEADER;
    }
    log.info("WWW-Authenticate header = '" + authenticateHeader + "'");
    
    super.init();
  }

  @Override
  protected boolean isPreconditionValid(WebdavRequest request, DavResource resource) {
    return !resource.exists() || request.matchesIfHeader(resource);
  }

  @Override
  public DavSessionProvider getDavSessionProvider() {
    return davService.getDavSessionProvider();
  }

  @Override
  public void setDavSessionProvider(DavSessionProvider provider) {
    davService.setSessionProvider(provider);
  }

  @Override
  public DavLocatorFactory getLocatorFactory() {
    return davService.getLocatorFactory(resourcePathPrefix);
  }

  @Override
  public void setLocatorFactory(DavLocatorFactory factory) {
    davService.setLocatorFactory(factory);
  }

  @Override
  public DavResourceFactory getResourceFactory() {
    return davService.getResourceFactory();
  }

  @Override
  public void setResourceFactory(DavResourceFactory factory) {
    davService.setResourceFactory(factory);
  }

  @Override
  public String getAuthenticateHeaderValue() {
    return authenticateHeader;
  }

  public static String getPathPrefix(ServletContext ctx) {
    return (String) ctx.getAttribute(CTX_ATTR_RESOURCE_PATH_PREFIX);
  }

}
