/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.ItemVisitor;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitions;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.DateValue;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: PropertyImpl.java 5862 2006-05-30 13:13:00Z peterit $
 */

public class PropertyImpl extends ItemImpl implements Property {

  protected int               type;

  protected Value[]           values;

  private PropertyDefinitions definitions;

  // private boolean multiValued;
  
  /**
   * New constructor
   * @param id
   * @param session
   */
  public PropertyImpl(String id, SessionImpl session) {
    super(id, session);
  }  

  PropertyImpl(PropertyData data, NodeImpl parentNode, int state, SessionImpl session
      ) throws RepositoryException, ConstraintViolationException {

    super(data, parentNode, state, session);
    // propertyData = (PropertyData)data;
    loadData(data);
  }

  void loadData(ItemData data) throws RepositoryException, ConstraintViolationException {

    // [PN] 10.04.06 Changed logic of copying value datas, now according stored order number 
    if (data instanceof NodeData)
      throw new RepositoryException("Load data: PropertyData is expected");

    PropertyData propertyData = (PropertyData) data;

    this.type = propertyData.getType();
    if (propertyData.getValues() != null) {
      // pack value datas
      List propValues = propertyData.getValues();
      Value[] newValues = new Value[propValues.size()];
      
      for (int i = 0; i < propValues.size(); i++) {
        ValueData valueData = (ValueData) propValues.get(i);
        // It makes new BaseValue object
        Value val = valueFactory.loadValue(valueData, type);
        
        if (val != null) {
          if (valueData.getOrderNumber() < newValues.length) {
            newValues[valueData.getOrderNumber()] = val;
          } else {
            throw new RepositoryException("loadData(): The property order number is out of size of multivalued property array. "+
                getPath());
          }
        }
      }
      this.values = newValues;
    }
    // this.data = data;
    initDefinitions();
  }

  protected void reloadPropertyData() throws RepositoryException {
    ItemData itemData = dataManager.getTransactManager().
      getItemData(getLocation().getInternalPath());

    if (itemData != null) { // || getPersistedVersion() > 0
      loadData(itemData);
    }  
  }
  
  private void loadProperty() throws RepositoryException {
    if (isRefreshToPersistedVersion()) {
      reloadPropertyData();
      setRefreshToPersistedVersion(false);
    }
  }
  
  private void initDefinitions() throws RepositoryException, ConstraintViolationException {

    NodeImpl parent = (NodeImpl) getParent();
    NodeType[] nodeTypes = parent.getAllNodeTypes();
    PropertyDefinitions defs = null;
    this.definitions = null;
    for (int i = 0; i < nodeTypes.length; i++) {
      
      defs = ((NodeTypeImpl) nodeTypes[i]).getPropertyDefinitions(getName());
      
      // [PN] 10.05.06 debug
//      log.info("initDefinitions() "+getName()+" nodeType:" + ((NodeTypeImpl) nodeTypes[i]).getName() + " defs:" 
//          + (defs != null ? (defs.getAnyDefinition() != null ? defs.getAnyDefinition().getName() : "[defs.getAnyDefinition():null]") : "[defs:null]"));      
//      PropertyDefinition[] allDefs = ((NodeTypeImpl) nodeTypes[i]).getPropertyDefinitions();
//      for (PropertyDefinition pdef: allDefs) {
//        log.info("initDefinitions() "+getName()+" pdef:" + pdef.getName() + " isMultiple:" + pdef.isMultiple() + " isMandatory:" + pdef.isMandatory());
//      }
      //----------------------
      
      if (defs.getAnyDefinition() != null) { // includes residual set
        definitions = defs;
        if (!((PropertyDefinitionImpl) defs.getAnyDefinition()).isResidualSet())
          break;
      }
    }

    if (definitions == null)
      throw new ConstraintViolationException("Definition for property " + getPath() + " not found.");
  }

  PropertyData storedPropertyData() {
    return (PropertyData) data;
  }

  public TransientItemData getActualItemData() throws RepositoryException {
    
    // [PN] 09.04.06
    loadProperty();
    
    TransientPropertyData newData = new TransientPropertyData(getLocation().getInternalPath(),
        internalUUID, persistedVersion, type, data.getParentUUID(), isMultiValued());
    List valueDatas = new ArrayList();
    if (values != null) {
      for (int i = 0; i < values.length; i++) {
        if (values[i] != null)
          valueDatas.add(((BaseValue) values[i]).getData());
      }
    }
    newData.setValues(valueDatas);
    // newData.setMultiValued(isMultiValued());
    return newData;
  }

  /**
   * @see javax.jcr.Property#getValue
   */
  public Value getValue() throws ValueFormatException, RepositoryException {
    
    // [PN] 09.04.06
    loadProperty();
    
    // if (definitions.getDefinition(false) == null || values.length > 1)
    if (getDefinition().isMultiple())
      throw new ValueFormatException("The property " + getPath() + " is multi-valued (6.2.4)");

    if (values != null && values.length == 0)
      throw new ValueFormatException("The single valued property " + getPath() + " is empty");
    // The object returned is a copy of the stored value and is immutable.
    return valueFactory.convert(values[0], this.type, true);

  }

  /**
   * @see javax.jcr.Property#getValues
   */
  public Value[] getValues() throws ValueFormatException, RepositoryException {
    
    // [PN] 09.04.06
    loadProperty();
    
    // if (definitions.getDefinition(true) == null)
    // Check property definition and life-state flag both
    if (!getDefinition().isMultiple() && !isMultiValued()) {
      throw new ValueFormatException("The property is single-valued (6.2.4)");
    }

    // The array returned is a copy of the stored values
    return valueFactory.convert(values, this.type, true);
    // return values;
  }

  /**
   * @see javax.jcr.Property#getString
   */
  public String getString() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getString();
    } catch (ValueFormatException e) {
      throw new ValueFormatException("PropertyImpl.getString() for " + getPath() + " failed: " + e);
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getString() for " + getPath() + " failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getDouble
   */
  public double getDouble() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getDouble();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getDouble() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getLong
   */
  public long getLong() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getLong();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getLong() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getStream
   */
  public InputStream getStream() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getStream();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getStream() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getDate
   */
  public Calendar getDate() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getDate();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getDate() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getBoolean
   */
  public boolean getBoolean() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getBoolean();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getBoolean() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getNode
   */
  public Node getNode() throws ValueFormatException, RepositoryException {
    try {
      String uuid = ((BaseValue) getValue()).getReference();
      return session.getNodeByUUID(uuid);
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getNode() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getLength
   */
  public long getLength() throws ValueFormatException, RepositoryException {

    BaseValue v = (BaseValue) getValue();
    if (v instanceof DateValue) {
      return ((DateValue) v).getString().length();
    } else {
      if (getType() == PropertyType.BINARY) {
        return v.getData().getAsByteArray().length;
      } else {
        String s = new String(v.getData().getAsByteArray());
        return s.length();
      }
    }
  }

  /**
   * @see javax.jcr.Property#getLengths
   */
  public long[] getLengths() throws ValueFormatException, RepositoryException {

    long[] lengths = new long[getValues().length];
    for (int i = 0; i < lengths.length; i++) {
      lengths[i] = ((BaseValue) getValues()[i]).getData().getAsByteArray().length;
    }
    return lengths;
  }

  /**
   * @see javax.jcr.Property#getDefinition
   */
  public PropertyDefinition getDefinition() throws RepositoryException {

    PropertyDefinition def = null;

    if (values == null || values.length == 0) {
      def = definitions.getAnyDefinition();
    }
    // else {
    // def = definitions.getDefinition(true);
    // }

    // TODO [PN] 30.05.06
    //PropertyData pdata = (PropertyData) data;
    //pdata.isMultiValued();
    
    if (def == null) { // && values.length == 1
      if (values.length == 1)
        def = definitions.getDefinition(false);
      if (def == null)
        def = definitions.getDefinition(true);
    }

    if (def == null) {
      throw new RepositoryException("FATAL: property definition is NULL " + getPath() + " "
          + values);
    } else {
      return def;
    }
  }

  /**
   * @see javax.jcr.Property#getType
   */
  public int getType() {
    return type;
  }

  /**
   * @see javax.jcr.Item#isModified
   */
  public boolean isModified() {
    return (getState() == ItemChangeState.UPDATED);
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Value value) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    // if (getDefinition().isMultiple()) {
    // throw new ValueFormatException("Assigning single Value to a
    // multiple-values property is imposible");
    // }

    Value[] newValues;
    int type = PropertyType.UNDEFINED;
    if (value != null) {
      newValues = new Value[1];
      newValues[0] = value;
      type = value.getType();
    } else {
      newValues = null;
    }
    doSetValue(newValues, false, type, ItemChangeState.UPDATED, true, true);

  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Value[] values) throws ValueFormatException, VersionException,
      LockException, ConstraintViolationException, RepositoryException {

    // if (!getDefinition().isMultiple()) {
    // throw new ValueFormatException("Assigning multiple Value values to a
    // single-valued property is imposible");
    // }

    int type = PropertyType.UNDEFINED;
    if (values != null && values.length > 0) {
      if (values[0] != null)
        type = values[0].getType();
      else
        type = PropertyType.UNDEFINED;
    }

    doSetValue(values, true, type, ItemChangeState.UPDATED, true, true);
  }
  
  // registerInDataManager,
  // doExternalValidation
  // TODO: Add ConstraintViolationException throw if the change would violate a
  // node type
  void doSetValue(Value[] propertyValues, boolean multiValue, int expectedType, int changeState,
      boolean registerInDataManager, boolean doExternalValidation) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    // log.debug("Definitions "+getPath()+" "+values.length+" "+definitions.dump());
    
    if (doExternalValidation) {
      NodeImpl parent = (NodeImpl) getParent();

      if (getDefinition().isProtected())
        throw new ConstraintViolationException("Can not set protected property " + getPath());

      if (!parent.isCheckedOut())
        throw new VersionException("Node " + parent.getPath()
            + " or its nearest ancestor is checked-in");

      // Check locking
      if (!parent.checkLocking())
        throw new LockException("Node " + parent.getPath() + " is locked ");
    }    

    // no such definition
    if (isMultiValued() != multiValue ) { //&& isMultiValued() == getDefinition().isMultiple()
      if (multiValue) {        
        throw new ValueFormatException("Can not assign multiple-values Value to a single-valued property " + getPath());
      } else {
        throw new ValueFormatException("Can not assign single-value Value to a multiple-valued property " + getPath());
      }
    }

    if (definitions.getDefinition(multiValue) == null)
      throw new ValueFormatException("Can not set value due to multiplicity constraint "
          + getPath());

    // Remove this property if values == null
    if (propertyValues == null) {
      if (getDefinition().isMandatory() || getDefinition().isProtected())
        throw new ConstraintViolationException(
            "Can not remove (by setting null value) protected or mandatory property " + getPath());

      remove();
      return;
    }

    // Remove null values and reorder values
    List<Value> valList = new ArrayList<Value>();
    for (int i = 0; i < propertyValues.length; i++) {
      
      if (propertyValues[i] != null) {        //[GA] 
         //&& ((BaseValue) propertyValues[i]).getData() != null) {
        
        valList.add(propertyValues[i]);
      }
    }

    Value[] newValues = new Value[valList.size()];
    
    for (int i = 0; i < newValues.length; i++) {
      Value v = valList.get(i);

      // reset order number
      //((BaseValue) v).getData().setOrderNumber(i);

      //newValues[i] = v;
      
      // Internal value SHOULD be BaseValue type
      // but incoming value SHOULD NOT
      // using convert() just to make new BaseValue object
      if(v instanceof BaseValue)
        newValues[i] = v;
      else
        newValues[i] = valueFactory.convert(v, v.getType(), true);

        //throw new RepositoryException("Incoming value is not BaseValue type. TODO any Value object should be supported!");
        //reset order number
      ((BaseValue) newValues[i]).getData().setOrderNumber(i);     
    }

    
    // Convert to required type if neccessary
    int requiredType = getDefinition().getRequiredType();
    int propType = requiredType;
    
    if (newValues.length > 0) {
      
      if (requiredType == PropertyType.UNDEFINED) {
        // requiredType = newValues[0].getType();
        if (expectedType == PropertyType.UNDEFINED) {
          expectedType = newValues[0].getType();
        }
        propType = expectedType;
      }
      
      // newValues = valueFactory.convert(newValues, propType, false);
      try {
        newValues = valueFactory.convert(newValues, propType, false);
      } catch (RepositoryException e) {
        if (expectedType != requiredType && expectedType != PropertyType.UNDEFINED) {
          if (e instanceof ValueFormatException)
            throw e;
          throw new ConstraintViolationException(
              "The change violates a requred node type with property '" + getPath()
                  + "'. A property must have type " + PropertyType.nameFromValue(requiredType)
                  + " found " + PropertyType.nameFromValue(expectedType)
                  + ". Converting is impossible: " + e.getMessage(), e);
        } else {
          throw e;
        }
      }
    }

    // Check value constraints
    checkValueConstraints(newValues);

    // Create (Reset) ValueData and PropertyType
    this.values = newValues;
    this.type = propType;

    // Register property with dataManager
    // [PN] 12.04.06
    setState(changeState);
    if (registerInDataManager) {
      // [PN] 12.04.06
      //setState(changeState);
      dataManager.update(this);
    }
    
  }
  /**
   * @return multiValued property of data field (PropertyData) it's a life-state
   *         property field which contains multiple-valued flag for value(s)
   *         data. Can be set in property creation time or from persistent
   *         storage.
   */
  public boolean isMultiValued() {
    return ((PropertyData) data).isMultiValued();
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(String value) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(value));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(InputStream stream) throws ValueFormatException, VersionException,
      LockException, ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(stream));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(double number) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(number));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(long number) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(number));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Calendar date) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(date));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(boolean b) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(b));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Node value) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    // Uuid uuid = new Uuid(value.getUUID());
    setValue(valueFactory.createValue(value));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(String[] values) throws ValueFormatException, VersionException,
      LockException, ConstraintViolationException, RepositoryException {

    Value[] strValues = null;
    if (values != null) {
      strValues = new Value[values.length];
      for (int i = 0; i < values.length; i++)
        strValues[i] = valueFactory.createValue(values[i]);
      // new StringValue(values[i]);
    }
    setValue(strValues);
  }

  // ////////////////// Item implementation ////////////////////

  /**
   * @see javax.jcr.Item#accept
   */
  public void accept(ItemVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

  /**
   * @see javax.jcr.Item#isNode
   */
  public boolean isNode() {
    return false;
  }

  // ////////////////////////////////

  public Value[] getValueArray() throws RepositoryException {
    
    // [PN] 09.04.06
    loadProperty();
    
    return values;
  }

  public String dump() {
    String vals = "Property " + getPath() + " values: ";
    try {  
      for (int i = 0; i < getValueArray().length; i++) {
        vals += new String(((BaseValue) getValueArray()[i]).getData().getAsByteArray()) + ";";
      }
    } catch(Exception e) {
      e.printStackTrace();
    }
    return vals;
  }

  private void checkValueConstraints(Value[] newValues) throws ConstraintViolationException,
      RepositoryException {

    String[] valConstraints = getDefinition().getValueConstraints();

    if (valConstraints != null && valConstraints.length != 0) {
      for (int j = 0; j < newValues.length; j++) {
        // do not use getString because of string consuming
        String strVal = new String(((BaseValue) newValues[j]).getData().getAsByteArray());
        // byte[] val = ((BaseValue) newValues[j]).getData().getAsByteArray();
        boolean invalid = true;
        for (int i = 0; i < valConstraints.length; i++) {
          switch (newValues[j].getType()) {
          case PropertyType.STRING:
            if (valConstraints[i].equals(strVal))
              invalid = false;            
          case PropertyType.REFERENCE:
            invalid = false;
          }
        }
        // log.warn("Value constraint is only supported for STRING property
        // (TODO) ");
        if (invalid)
              throw new ConstraintViolationException("Can not set value " + strVal + " to " + getPath()
              + " due to value constraints ");
      }
    }
  }

  //----------------------- Object.equals() ----------------------

  /**
   * @author [PN] 18.04.06
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (obj instanceof PropertyImpl) {
      try {
        // by path
        return getLocation().equals(((PropertyImpl) obj).getLocation());
      } catch(Exception e) {
        return false;
      }
    }
    return false;
  }  
  
  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    try {
      return getLocation().getAsString(false).hashCode();
    } catch(Exception e) {
      return super.hashCode();
    }
  }   
}