/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import javax.jcr.Credentials;
import javax.jcr.RepositoryException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.jcr.core.XASession;
import org.exoplatform.services.jcr.impl.dataflow.session.TransactionableDataManager;
import org.exoplatform.services.transaction.TransactionException;
import org.exoplatform.services.transaction.TransactionService;
import org.objectweb.transaction.jta.ResourceManagerEvent;
/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */

public class XASessionImpl extends SessionImpl implements XASession, XAResource, ResourceManagerEvent {
   
  private TransactionService tService;
  private TransactionableDataManager transactManager;
  private int txTimeout;

  XASessionImpl(String workspaceName, Credentials credentials, ExoContainer container,
      TransactionService tService)  throws RepositoryException {
    super(workspaceName, credentials, container);
    txTimeout = tService.getDefaultTimeout();
    transactManager = nodesManager.getTransactManager();
    this.tService = tService;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.core.XASession#getXAResource()
   */
  public XAResource getXAResource() {
    return this;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.core.XASession#delistResource()
   */
  public void delistResource() throws XAException {
    try {
      tService.delistResource(this);
    } catch (RollbackException e) {
      throw new XAException(e.getMessage());
    } catch (SystemException e) {
      throw new XAException(e.getMessage());
    }  
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.core.XASession#reenlistResource()
   */
  public void enlistResource() throws XAException {
    try {
      tService.enlistResource(this);
    } catch (RollbackException e) {
      throw new XAException(e.getMessage());
    } catch (SystemException e) {
      throw new XAException(e.getMessage());
    }  
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#commit(javax.transaction.xa.Xid, boolean)
   */
  public void commit(Xid xid, boolean onePhase) throws XAException {
    try {
      transactManager.commit();
    } catch (TransactionException e) {
      throw new XAException(XAException.XA_RBOTHER);
    }
    log.debug("Commit "+xid);
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#end(javax.transaction.xa.Xid, int)
   */
  public void end(Xid xid, int flags) throws XAException {
    log.debug("End "+xid);
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#forget(javax.transaction.xa.Xid)
   */
  public void forget(Xid xid) throws XAException {
    // TODO 
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#getTransactionTimeout()
   */
  public int getTransactionTimeout() throws XAException {
    return txTimeout;
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#isSameRM(javax.transaction.xa.XAResource)
   */
  public boolean isSameRM(XAResource resource) throws XAException {
    if (resource instanceof XASessionImpl) {
      XASessionImpl session = (XASessionImpl) resource;
      return getUserID().equals(session.getUserID());
    }
    return false;
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#prepare(javax.transaction.xa.Xid)
   */
  public int prepare(Xid xid) throws XAException {
    log.debug("Prepare "+xid);
    return XA_OK;
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#recover(int)
   */
  public Xid[] recover(int xid) throws XAException {
    // TODO 
    return null;
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#rollback(javax.transaction.xa.Xid)
   */
  public void rollback(Xid xid) throws XAException {
    transactManager.rollback();
    log.debug("Rollback "+xid);
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#setTransactionTimeout(int)
   */
  public boolean setTransactionTimeout(int seconds) throws XAException {
    try {
      tService.setTransactionTimeout(seconds);
    } catch (SystemException e) {
      throw new XAException(e.getMessage());
    }
    this.txTimeout = seconds;
    return true;
  }

  /* (non-Javadoc)
   * @see javax.transaction.xa.XAResource#start(javax.transaction.xa.Xid, int)
   */
  public void start(Xid xid, int flags) throws XAException {
    transactManager.start();
    log.debug("Start "+xid);
  }

  @Override
  public void logout() {
    super.logout();
    try {
      delistResource();
    } catch (XAException e) {
      e.printStackTrace();
    }
  }
  
  public void enlistConnection(Transaction transaction) throws javax.transaction.SystemException  {
    try {
      enlistResource();
    } catch (IllegalStateException e) {
      throw new SystemException(e.getMessage());
    } catch (XAException e) {
      throw new SystemException(e.getMessage());
    } 
  }


}
