/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.impl.core.observation;

import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NodeType;
import javax.jcr.observation.Event;
import javax.jcr.observation.EventListener;
import javax.jcr.observation.EventListenerIterator;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.dataflow.ItemsPersistenceListener;
import org.exoplatform.services.jcr.impl.dataflow.persistent.WorkspaceDataManager;
import org.exoplatform.services.jcr.impl.util.EntityCollection;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: ActionLauncher.java 5466 2006-05-10 11:41:48Z peterit $
 */

public class ActionLauncher implements ItemsPersistenceListener {

  private static Log log = ExoLogger.getLogger("jcr.ActionLauncher");

  private ObservationManagerRegistry observationRegistry;

  private WorkspaceDataManager workspaceDataManager;

  public ActionLauncher(ObservationManagerRegistry registry,
      LogService logService, WorkspaceDataManager workspaceDataManager) {
    this.observationRegistry = registry;
    //log = (ExoLog) logService.getLog(this.getClass());
    this.workspaceDataManager = workspaceDataManager;
    workspaceDataManager.addItemPersistenceListener(this);
  }

  public void onBatchSaveItems(List itemStates, SessionImpl session) {

    EventListenerIterator eventListeners = observationRegistry
        .getEventListeners();

    while (eventListeners.hasNext()) {

      EventListener listener = eventListeners.nextEventListener();
      ListenerCriteria c = observationRegistry.getListenerFilter(listener);

      EntityCollection events = new EntityCollection();

      for (Iterator i = itemStates.iterator(); i.hasNext();) {
        ItemState itemState = (ItemState) i.next(); 
        ItemData item = itemState.getData();

        try {
          int[] eventTypes = getEventTypes(itemState.getState(), item, c);

          for (int j = 0; j < eventTypes.length; j++) {

            int evenType = eventTypes[j];
            
            if (isTypeMatch(c, evenType) && isPathMatch(c, item)
                && isUUIDMatch(c, item) && isNodeTypeMatch(c, item)
                && isSessionMatch(c, session)) {
              //events.add(new EventImpl(evenType, item, session));
              SessionImpl ses = c.getSession();
              String path = session.getLocationFactory().
                createJCRPath(item.getQPath()).getAsString(false);
              events.add(new EventImpl(evenType, path, ses.getUserID()));
            }
          }
        } catch (RepositoryException e) {
          log.error("Can not fire ActionLauncher.onSaveItems() for "
              + item.getQPath().getAsString() + " reason: " + e);
        }

      }
      if (events.size() > 0) {
        // no events - no onEvent() action, it's a play for TCK   
        listener.onEvent(events);
      }
    }
  }

  public void onSaveItems(List itemStates, SessionImpl session) {
  }

  // ---------------------------------

  private boolean isTypeMatch(ListenerCriteria criteria, int state) {
    return (criteria.getEventTypes() & state) > 0;
  }

  private boolean isSessionMatch(ListenerCriteria criteria, SessionImpl session) {
    if (criteria.getNoLocal() && criteria.getSession().equals(session))
      return false;
    return true;
  }

  private boolean isPathMatch(ListenerCriteria criteria, ItemData item) {
    if (criteria.getAbsPath() == null)
      return true;
    try {
      InternalQPath cLoc = criteria.getSession().getLocationFactory()
          .parseAbsPath(criteria.getAbsPath()).getInternalPath();
      // 8.3.3 Only events whose associated parent node is at absPath (or
      // within its subtree, if isDeep is true) will be received.

      // assotiated parent is node itself for node and parent for property ????
      /*
       if(!item.isNode()) { 
       boolean match = item.getQPath().makeParentPath().equals(cLoc);
       if(!match && criteria.isDeep())
       match = item.getQPath().makeParentPath().isDescendantOf(cLoc, false);
       return match;
       } else {
       boolean match = item.getQPath().equals(cLoc);
       if(!match && criteria.isDeep())
       match = item.getQPath().isDescendantOf(cLoc, false);
       return match;
       }
       */
      InternalQPath itemPath = item.getQPath();
      if (item.isNode()) {
        boolean isAtAbsPathNosde = cLoc.equals(itemPath);
        if (isAtAbsPathNosde) {
          // This is a associated parent node
          return true; //state == Event.NODE_ADDED          
        } else if (criteria.isDeep()) {
          // For descendant
          return itemPath.isDescendantOf(cLoc, false);
        } else {
          // no path match
          return false;
        }
      } else {
        // property is descendant always
        return itemPath.isDescendantOf(cLoc, !criteria.isDeep());
      }
    } catch (RepositoryException e) {
      return false;
    }
  }

  private boolean isUUIDMatch(ListenerCriteria criteria, ItemData item) {

    if (criteria.getUuid() == null)
      return true;

    // assotiated parent is node itself for node and parent for property ????
    for (int i = 0; i < criteria.getUuid().length; i++) {
      if (item.isNode() && criteria.getUuid()[i].equals(item.getUUID()))
        return true;
      else if (!item.isNode()
          && criteria.getUuid()[i].equals(item.getParentUUID()))
        return true;
    }
    return false;

  }

  /*
   private boolean isUUIDMatch(ListenerCriteria criteria, ItemData item) {
   
   if (criteria.getUuid() == null)
   return true;
   
   if (!item.isNode())
   return false;
   NodeData node = (NodeData) item;
   if (!isType(node, "mix:referenceable", criteria.getSession()))
   return false;

   for (int i = 0; i < criteria.getUuid().length; i++) {
   
   if (criteria.getUuid()[i].equals(node.getUUID()))
   return true;
   }
   return false;
   
   }

   private boolean isNodeTypeMatch(ListenerCriteria criteria, ItemData item) {
   if (criteria.getNodeTypeName() == null)
   return true;
   if (!item.isNode())
   return false;
   NodeData node = (NodeData) item;
   
   for (int i = 0; i < criteria.getNodeTypeName().length; i++) {
   if (isType(node, criteria.getNodeTypeName()[i], criteria.getSession()))
   return true;
   }
   return false;
   }
   */
  private boolean isNodeTypeMatch(ListenerCriteria criteria, ItemData item)
      throws RepositoryException {
    if (criteria.getNodeTypeName() == null)
      return true;

    NodeData node;

    node = (NodeData) workspaceDataManager.getItemData(item.getParentUUID());

    if (node == null) {
      return false;
    }

    for (int i = 0; i < criteria.getNodeTypeName().length; i++) {
      if (isType(node, criteria.getNodeTypeName()[i], criteria.getSession()))
        return true;
    }
    return false;
  }

  private boolean isType(NodeData data, String typeName, SessionImpl session) {
    InternalQName primaryType = data.getPrimaryTypeName();
    //    System.out.println("TYPE>>>> "+primaryType+" "+data.getQPath().getAsString());

    InternalQName[] mixinTypes = data.getMixinTypeNames();
    int mixLength = 0;
    if (mixinTypes != null)
      mixLength = mixinTypes.length;
    InternalQName[] allTypes = new InternalQName[mixLength + 1];
    for (int i = 0; i < mixLength; i++)
      allTypes[i] = mixinTypes[i];
    allTypes[mixLength] = primaryType;
    for (int i = 0; i < allTypes.length; i++) {
      NodeType nodeType;
      try {
        nodeType = ((NodeTypeManagerImpl) session.getWorkspace()
            .getNodeTypeManager()).findNodeType(allTypes[i]);
      } catch (RepositoryException e) {
        return false;
      }
      if (nodeType.isNodeType(typeName))
        return true;
    }
    return false;
  }

  /**
   * @param state
   * @param isNode
   * @return event types that can be generated by this change 
   * (Note there can be more than one events)
   * @throws RepositoryException
   */
  private int[] getEventTypes(int state, ItemData item, ListenerCriteria c)
      throws RepositoryException {
    if (item.isNode()) {
      if (state == ItemChangeState.ADDED || state == ItemChangeState.AUTO_ADDED)
        return new int[] { Event.NODE_ADDED };
      else if (state == ItemChangeState.DELETED
          || state == ItemChangeState.AUTO_DELETED)
        return new int[] { Event.NODE_REMOVED };
      else if (state == ItemChangeState.UPDATED)
        // [PN] 07.02.06
        if (item.isNode()) {
          NodeData node = (NodeData) item;
          // This is a oredered node (in this session we hope!!!), 
          // 8.3.7.8 Re-ordering a set of Child Nodes
          // When an orderBefore(A, B) is performed, an implementation
          // must generate a NODE_REMOVED for node A and a NODE_ADDED for
          // node A.
          if (node.getOrderDirection() == Event.NODE_ADDED) {
            // Same name sibs reordered source (added)
            return new int[] { Event.NODE_ADDED };
          } else if (node.getOrderDirection() == Event.NODE_REMOVED) {
            // Same name sibs reordered destination (removed)
            return new int[] { Event.NODE_REMOVED };
          } else if (node.getOrderDirection() == (Event.NODE_ADDED | Event.NODE_REMOVED)) {
            // Different nodes ordered
            return new int[] { Event.NODE_ADDED, Event.NODE_REMOVED };
          }
        }
      return new int[] {};
    } else {
      if (state == ItemChangeState.ADDED || state == ItemChangeState.AUTO_ADDED)
        return new int[] { Event.PROPERTY_ADDED };
      else if (state == ItemChangeState.DELETED
          || state == ItemChangeState.AUTO_DELETED)
        return new int[] { Event.PROPERTY_REMOVED };
      else if (state == ItemChangeState.UPDATED)
        return new int[] { Event.PROPERTY_CHANGED };
    }
    throw new RepositoryException("Event type could not be defined! state: "
        + ItemChangeState.getStateName(state) + " isNode: " + item.isNode());
  }

}
