/*
 * Copyright 2004-2005 The Apache Software Foundation or its licensors,
 *                     as applicable.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.exoplatform.services.jcr.impl.core.query;

import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.dataflow.persistent.WorkspaceDataManager;

import sun.management.FileSystem;

/**
 * Acts as an argument for the {@link QueryHandler} to keep the interface
 * stable. This class provides access to the environment where the query
 * handler is running in.
 */
public class QueryHandlerContext {

    /**
     * The node type registry of the repository
     */
    private final LocationFactory sysLocationFactory;

    /**
     * The UUID of the root node.
     */
    private String rootUUID;
    
    private WorkspaceDataManager dataManager;


    /**
     * Creates a new context instance.
     *
     * @param fs         a {@link FileSystem} this <code>QueryHandler</code> may
     *                   use to store its index. If no <code>FileSystem</code>
     *                   has been configured <code>fs</code> is
     *                   <code>null</code>.
     * @param stateMgr   provides persistent item states.
     * @param rootUUID   the uuid of the root node.
     * @param ntRegistry the node type registry.
     */
    public QueryHandlerContext(
                               WorkspaceDataManager dataManager,
                               //String rootUUID,
                               LocationFactory sysLocationFactory
                               ) {
        this.dataManager = dataManager;
        //this.rootUUID = rootUUID;
        this.rootUUID = Constants.ROOT_UUID;
        this.sysLocationFactory = sysLocationFactory;
    }

    
    public WorkspaceDataManager getDataManager() {
      return dataManager;
    }


    /**
     * Returns the UUID of the root node.
     * @return the UUID of the root node.
     */
    public String getRootUUID() {
        return rootUUID;
    }


    /**
     * Destroys this context and releases resources.
     */
    public void destroy() {
//        ntRegistry.removeListener(propRegistry);
    }
    
    /**
     * @return Returns the sysLocationFactory.
     */
    public LocationFactory getSysLocationFactory() {
      return sysLocationFactory;
    }
}
