/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;

import javax.jcr.RepositoryException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.services.database.jdbc.DBSchemaCreator;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.jcr.impl.storage.value.ValueStoragePluginProvider;
import org.exoplatform.services.jcr.storage.WorkspaceStorageConnection;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: JDBCWorkspaceDataContainer.java 6386 2006-06-22 16:11:45Z peterit $
 */
public class JDBCWorkspaceDataContainer extends
    WorkspaceDataContainerBase {

  protected static Log log = ExoLogger.getLogger("jcr.JDBCWorkspaceDataContainer");
  
  protected String containerName;
  protected String sourceName;
  protected boolean multiDB;
  private DataSource ds;
  private ValueStoragePluginProvider valueStorageProvider;

  /**
   * Constructor with no value storage plugins configured
   * @param wsConfig
   * @param dbCreator
   * @throws RepositoryConfigurationException
   * @throws NamingException
   */
  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, 
      DBSchemaCreator dbCreator) 
  throws RepositoryConfigurationException, NamingException, RepositoryException, IOException {
    this(wsConfig, dbCreator, null);
  }
  
  /**
   * Constructor with value storage plugins
   * @param wsConfig
   * @param dbCreator
   * @param valueStrorageProvider
   * @throws RepositoryConfigurationException
   * @throws NamingException
   */
  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, 
      DBSchemaCreator dbCreator,
      ValueStoragePluginProvider valueStorageProvider) 
  throws RepositoryConfigurationException, NamingException, RepositoryException, IOException {
    
    this.containerName = wsConfig.getName();
    this.sourceName = wsConfig.getContainer().getParameterValue("sourceName");
//    this.valueStorageProvider = valueStorageProvider;
    this.valueStorageProvider = ValueStoragePluginProvider.getInstance(wsConfig);

    
    System.out.println(">>>>>>>>>>>.STORAGE provider >>>>>>>>>>>."+valueStorageProvider+
        " "+wsConfig.getContainer().getValueStorages());
    
    multiDB = Boolean.parseBoolean(
        wsConfig.getContainer().getParameterValue("multi-db"));
    
    Context context = new InitialContext();
    ds = (DataSource) context.lookup(sourceName);
    
    log.info("JDBCWorkspaceDataContainer (cEntry): " + containerName + ", dataSource: " + sourceName);
  }

  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#openConnection()
   */
  public WorkspaceStorageConnection openConnection() throws RepositoryException {
    
    try {

      Connection conn = ds.getConnection();
      
      // Fix for Sybase jConnect JDBC driver bug. 
      // Which throws SQLException(JZ016: The AutoCommit option is already set to false)
      // if conn.setAutoCommit(false) called twise or more times with value 'false'.
      if (conn.getAutoCommit()) {
        conn.setAutoCommit(false);
      }
      
      if (multiDB) 
        return new MultiDbJDBCConnection(conn, containerName, valueStorageProvider);
      else 
        return new SingleDbJDBCConnection(conn, containerName, valueStorageProvider);

    } catch (SQLException e) {
      throw new RepositoryException(e);
    }
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getName()
   */
  public String getName() {
    return containerName;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getInfo()
   */
  public String getInfo() {
    String str = "Info: JDBC based JCR Workspace Data container \n";
    //str += "\tName: " + containerName + "\n";
    //str += "\tHibernate configuration: \n";
    //str += "\tDatasource name (hibernate.datasource.name): ???"
    //    + "\n";
    return str;
  }

//  public boolean isMultiDB() {
//    return multiDB; 
//  }
  

}