/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.io.ByteArrayInputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.jcr.PropertyType;

import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.storage.value.ValueStoragePluginProvider;

/**
 * Created by The eXo Platform SARL
 * 27.04.2006
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: SingleDbJDBCConnection.java 6375 2006-06-22 09:17:39Z geaz $
 */
public class SingleDbJDBCConnection extends JDBCStorageConnection {

  static {
    
    JCR_FK_NODE_PARENT = "JCR_FK_SNODE_PARENT";
    JCR_FK_NODE_ITEM = "JCR_FK_SNODE_ITEM";
    JCR_FK_PROPERTY_NODE = "JCR_FK_SPROPERTY_N";
    JCR_FK_PROPERTY_ITEM = "JCR_FK_SPROPERTY_I";
    JCR_FK_VALUE_PROPERTY = "JCR_FK_SVALUE_PROP";
    
    FIND_ITEM_BY_ID = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I LEFT JOIN JCR_SNODE N ON I.ID=N.ID LEFT JOIN JCR_SPROPERTY P ON I.ID=P.ID"
      + " where I.ID=? and I.CONTAINER_NAME=?";

    FIND_ITEM_BY_PATH = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I LEFT JOIN JCR_SNODE N ON I.ID=N.ID LEFT JOIN JCR_SPROPERTY P ON I.ID=P.ID"
      + " where I.PATH=? and I.CONTAINER_NAME=?";
    
    FIND_CHILDITEM_BY_PATH = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I left join JCR_SNODE N on (I.ID=N.ID and N.PARENT_ID=?) left join JCR_SPROPERTY P on (I.ID=P.ID and P.PARENT_ID=?)"
      + " where I.PATH=? and I.CONTAINER_NAME=?";

    //FIND_NODE_BY_ID = "select * from JCR_SNODE where ID=?";
    //FIND_PROPERTY_BY_ID = "select * from JCR_SPROPERTY where ID=?";
    
    FIND_PROPERTIES_BY_VALUE = "select I.*, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED, V.ORDER_NUM as VORDER_NUM, V.PROPERTY_ID as VPROPERTY_ID, V.DATA as VDATA"
      + " from JCR_SITEM I, JCR_SPROPERTY P, JCR_SVALUE V"
      + " where I.ID=P.ID and P.ID=V.PROPERTY_ID and P.TYPE=" + PropertyType.REFERENCE
      + " and V.DATA=? and I.CONTAINER_NAME=?"
      + " order by I.ID";
    
    FIND_VALUES_BY_PROPERTYID = "select * from JCR_SVALUE where PROPERTY_ID=? order by ORDER_NUM";
    
    FIND_NODES_BY_PARENTID = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID" 
      + " from JCR_SITEM I, JCR_SNODE N"
      + " where I.ID=N.ID and N.PARENT_ID=?" 
      + " order by I.ID";
    
    // [PN] 22.05.06
    FIND_NODES_IDS_BY_PARENTID = "select I.ID" 
      + " from JCR_SITEM I, JCR_SNODE N"
      + " where I.ID=N.ID and N.PARENT_ID=?"
      + " group by I.ID"
      + " order by I.ID";
    
    FIND_PROPERTIES_BY_PARENTID = "select I.*, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I, JCR_SPROPERTY P"
      + " where I.ID=P.ID and P.PARENT_ID=?" 
      + " order by I.ID";
    
    FIND_NODESCOUNT_BY_PARENTID = "select count(*) from JCR_SNODE where PARENT_ID=?";
    FIND_PROPERTIESCOUNT_BY_PARENTID = "select count(*) from JCR_SPROPERTY where PARENT_ID=?";
    
    INSERT_ITEM = "insert into JCR_SITEM(ID, PATH, VERSION, CONTAINER_NAME) VALUES(?,?,?,?)";
    INSERT_NODE = "insert into JCR_SNODE(ID, ORDER_NUM, PARENT_ID) VALUES(?,?,?)";
    INSERT_PROPERTY = "insert into JCR_SPROPERTY(ID, TYPE, MULTIVALUED, PARENT_ID) VALUES(?,?,?,?)";
    INSERT_VALUE = "insert into JCR_SVALUE(DATA, ORDER_NUM, PROPERTY_ID) VALUES(?,?,?)";

    UPDATE_ITEM = "update JCR_SITEM set VERSION=? where ID=? and CONTAINER_NAME=?";
    UPDATE_NODE = "update JCR_SNODE set ORDER_NUM=? where ID=?";
    UPDATE_PROPERTY = "update JCR_SPROPERTY set TYPE=? where ID=?";
    
    DELETE_ITEM = "delete from JCR_SITEM where ID=? and CONTAINER_NAME=?";
    DELETE_NODE = "delete from JCR_SNODE where ID=?";
    DELETE_PROPERTY = "delete from JCR_SPROPERTY where ID=?";
    DELETE_VALUE = "delete from JCR_SVALUE where PROPERTY_ID=?";
   }
  
  SingleDbJDBCConnection(Connection dbConnection, String containerName, ValueStoragePluginProvider valueStorageProvider) throws SQLException {
     super(dbConnection, containerName, valueStorageProvider);
   }

  @Override
  protected void addNodeRecord(NodeData data) throws SQLException {
    
    String uuid = id2cid(data.getUUID());
    
    insertItem.setString(1, uuid);
    insertItem.setString(2, data.getQPath().getAsString());
    insertItem.setInt(3, data.getPersistedVersion());
    insertItem.setString(4, containerName);
    insertItem.executeUpdate();

    insertNode.setString(1, uuid);
    insertNode.setInt(2, data.getOrderNumber());
    insertNode.setString(3, data.getParentUUID() != null ? id2cid(data.getParentUUID()) : null);
    insertNode.executeUpdate();    
  }

  @Override
  protected void addPropertyRecord(PropertyData data) throws SQLException {
    
    String uuid = id2cid(data.getUUID());
    
    insertItem.setString(1, uuid);
    insertItem.setString(2, data.getQPath().getAsString());
    insertItem.setInt(3, data.getPersistedVersion());
    insertItem.setString(4, containerName);
    insertItem.executeUpdate();

    insertProperty.setString(1, uuid);
    insertProperty.setInt(2, data.getType());
    insertProperty.setBoolean(3, data.isMultiValued());
    insertProperty.setString(4, id2cid(data.getParentUUID()));
    insertProperty.executeUpdate();
    
//    addValueRecords(data);
  }

  @Override
  protected void addValueRecords(PropertyData data) throws SQLException {
    List values = data.getValues();
    for(int i=0; i<values.size(); i++) {
      ValueData val = (ValueData) values.get(i);
      // [PN] 13.06.06
      //insertValue.setBytes(1, val.getAsByteArray());
      ByteArrayInputStream bs = new ByteArrayInputStream(val.getAsByteArray());
      insertValue.setBinaryStream(1, bs, bs.available());
      
      // [PN] 26.04.06 Natural values order using
      val.setOrderNumber(i);
      insertValue.setInt(2, val.getOrderNumber());
      
      insertValue.setString(3, id2cid(data.getUUID()));
      insertValue.executeUpdate();
    }
  }  

  @Override
  protected int deleteItemByUUID(String uuid) throws SQLException {
    deleteItem.setString(1, id2cid(uuid));
    deleteItem.setString(2, containerName);
    return deleteItem.executeUpdate();
  }

  @Override
  protected int deleteNodeByUUID(String uuid) throws SQLException {
    deleteNode.setString(1, id2cid(uuid));
    return deleteNode.executeUpdate();
  }

  @Override
  protected int deletePropertyByUUID(String uuid) throws SQLException {
    deleteProperty.setString(1, id2cid(uuid));
    return deleteProperty.executeUpdate();
  }

  @Override
  protected int deleteValueByUUID(String uuid) throws SQLException {
    deleteValue.setString(1, id2cid(uuid));
    return deleteValue.executeUpdate();
  }

  @Override
  protected ResultSet findChildNodesByParentUUID(String parentUUID) throws SQLException {
    findNodesByParentId.setString(1, id2cid(parentUUID));
    return findNodesByParentId.executeQuery();
  }
  
  @Override
  protected ResultSet findChildNodesIdsByParentUUID(String parentUUID) throws SQLException {
    findNodesIdsByParentId.setString(1, id2cid(parentUUID));
    return findNodesIdsByParentId.executeQuery();
  }
  

  @Override
  protected ResultSet findChildPropertiesByParentUUID(String parentUUID) throws SQLException {
    findPropertiesByParentId.setString(1, id2cid(parentUUID));
    return findPropertiesByParentId.executeQuery();
  }

  @Override
  protected ResultSet findItemByPath(String path) throws SQLException {
    findItemByPath.setString(1, path);
    findItemByPath.setString(2, containerName);
    return findItemByPath.executeQuery();
  }
  
  @Override
  protected ResultSet findItemByPath(String parentId, String path) throws SQLException {
    
    String parentUuid = id2cid(parentId);
    
    findChildItemByPath.setString(1, id2cid(parentUuid));
    findChildItemByPath.setString(2, id2cid(parentUuid));
    findChildItemByPath.setString(3, path);
    findChildItemByPath.setString(4, containerName);
    return findChildItemByPath.executeQuery();
  }

  @Override
  protected ResultSet findItemByUUID(String uuid) throws SQLException {
    findItemById.setString(1, id2cid(uuid));
    findItemById.setString(2, containerName);
    return findItemById.executeQuery();
  }

  @Override
  protected ResultSet findReferences(String uuid) throws SQLException {
    //findPropertiesByValue.setString(1, uuid); // we have to use uuid from item 
    findPropertiesByValue.setBytes(1, uuid.getBytes()); // we have to use uuid from item
    findPropertiesByValue.setString(2, containerName);
    return findPropertiesByValue.executeQuery();
  }

  @Override
  protected ResultSet findValuesByPropertyId(String propertyId) throws SQLException {
    findValuesByPropertyId.setString(1, id2cid(propertyId));
    return findValuesByPropertyId.executeQuery();
  }

  @Override
  protected ResultSet getChildNodesCountByParentUUID(String parentUUID) throws SQLException {
    findNodesCountByParentId.setString(1, id2cid(parentUUID));
    return findNodesCountByParentId.executeQuery();
  }

  @Override
  protected ResultSet getChildPropertiesCountByParentUUID(String parentUUID) throws SQLException {
    findPropertiesCountByParentId.setString(1, id2cid(parentUUID));    
    return findPropertiesCountByParentId.executeQuery();
  }

  @Override
  protected int updateItemVersionByUUID(int versionValue, String uuid) throws SQLException {
    updateItem.setInt(1, versionValue);
    updateItem.setString(2, id2cid(uuid));
    updateItem.setString(3, containerName);
    return updateItem.executeUpdate();
  }
  
  @Override
  protected int updateNodeOrderNumbByUUID(int orderNumb, String uuid) throws SQLException {
    updateNode.setInt(1, orderNumb);
    updateNode.setString(2, id2cid(uuid));
    return updateNode.executeUpdate();
  }
  
  @Override
  protected int updatePropertyTypeByUUID(int type, String uuid) throws SQLException {
    updateProperty.setInt(1, type);
    updateProperty.setString(2, id2cid(uuid));
    return updateProperty.executeUpdate();
  }
}
