/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.storage.rdb.container;

import java.util.List;

import javax.jcr.InvalidItemStateException;
import javax.jcr.ItemExistsException;
import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBPropertyRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.NodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.PropertyRecord;
import org.exoplatform.services.log.LogService;
import org.hibernate.NonUniqueObjectException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 20.03.2006
 * @version $Id: HibernateMultiStorageConnection.java 5307 2006-04-27 13:55:57Z peterit $
 */
public class HibernateMultiStorageConnection extends HibernateStorageConnection {

  protected static String FIND_ITEM_BY_PATH = "from MultiRDBItemRecord as item where item.path=?";

  protected static String FIND_NODE_BY_PATH = "from MultiRDBNodeRecord as item where item.path=?";

  protected static String FIND_PROPERTY_BY_PATH = "from MultiRDBPropertyRecord as item where item.path=?";

  protected static String FIND_PROPSVALUES_BY_TYPE_AND_VALUE = "select distinct prop from MultiRDBValueRecord as val, MultiRDBPropertyRecord as prop inner join val.property where prop.valuesType=? and val.data=?";

  protected static String FIND_PROPS_BY_VALUE = "from MultiRDBValueRecord as val where val.data=?";

  protected static String FIND_NODES_BY_PARENTUUID = "from MultiRDBNodeRecord as item where item.parentNode=?";
  
  protected static String FIND_PROPERTIES_BY_PARENTUUID = "from MultiRDBPropertyRecord as item where item.parentNode=?";

  protected static String FIND_NODESCOUNT_BY_PARENTUUID = "select count(*) from MultiRDBNodeRecord as item where item.parentNode=?";
  
  protected static String FIND_PROPERTIESCOUNT_BY_PARENTUUID = "select count(*) from MultiRDBPropertyRecord as item where item.parentNode=?";
  
  public HibernateMultiStorageConnection(String sourceName, 
      SessionFactory sessionFactory) {
   
    super(sourceName, sessionFactory);
    log.debug("HibernateMultyStorageConnection (cEntry) " 
        + " hibernate container = " + sourceName);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#addNodeRecord(org.exoplatform.services.jcr.datamodel.NodeData, org.hibernate.Session)
   */
  synchronized protected NodeRecord addNodeRecord(NodeData data,
      Session hibernateSession) throws RepositoryException,
      ItemExistsException, InvalidItemStateException {

    //log.debug("RDB Container is adding Node: " + data.getQPath().getAsString()
    //    + " index " + data.getQPath().getIndex() + " UUID: " + data.getUUID());   
    
    NodeRecord parent = findParentNodeRecord(data);
    MultiRDBNodeRecord nodeRecord = new MultiRDBNodeRecord();
    nodeRecord.setId(data.getUUID());
    nodeRecord.setPath(data.getQPath().getAsString());
    nodeRecord.setParentNode(parent);
    nodeRecord.setOrderNumber(data.getOrderNumber());   
    
    if (parent != null) // not root
      parent.addChildNode(nodeRecord);

    try {
      hibernateSession.persist(nodeRecord);
    } catch (NonUniqueObjectException e) {
      String sameUuidPath = "";
      try {
        NodeRecord sameUuid = findNodeByUUID(data.getUUID(), hibernateSession);
        if (sameUuid != null) { 
          sameUuidPath = sameUuid.getPath();
        }
      } catch(Exception e1) {
        log.info("addNodeRecord(): same uuid node search ERROR: " + data.getUUID(), e); 
      }
      throw new ItemExistsException("Item with UUID "+data.getUUID() + "  " + sameUuidPath + "  already exists");
    } finally {
      //toLog("  addNodeRecord(NodeData) fin: " + hibernateSession);
    }

    return nodeRecord;

  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#addPropertyRecord(org.exoplatform.services.jcr.datamodel.PropertyData, org.exoplatform.services.jcr.impl.storage.rdb.container.data.NodeRecord, org.hibernate.Session)
   */
  synchronized protected PropertyRecord addPropertyRecord(PropertyData prop,
      NodeRecord parent, Session hibernateSession)
      throws RepositoryException, ItemExistsException {

    MultiRDBPropertyRecord propertyRecord = new MultiRDBPropertyRecord();
    propertyRecord.setId(prop.getUUID());
    propertyRecord.setPath(prop.getQPath().getAsString());
    propertyRecord.setType(prop.getType());
    propertyRecord.setValues(prop.getValues());
    propertyRecord.setMultiValued(prop.isMultiValued());

    parent.addProperty(propertyRecord);

    hibernateSession.persist(propertyRecord);
    
    log.debug("RDB Container added: " + propertyRecord);

    return propertyRecord;

  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findItemByPath(java.lang.String, org.hibernate.Session)
   */
  protected ItemRecord findItemByPath(String path, Session hibernateSession) {
    
    MultiRDBItemRecord r = (MultiRDBItemRecord) hibernateSession.createQuery(FIND_ITEM_BY_PATH)
      .setString(0, path).uniqueResult();
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findItemByUUID(java.lang.String, org.hibernate.Session)
   */
  protected ItemRecord findItemByUUID(String uuid, Session hibernateSession) {
    
    MultiRDBItemRecord r = (MultiRDBItemRecord) hibernateSession.get(MultiRDBItemRecord.class,
        uuid);
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findNodeByPath(java.lang.String, org.hibernate.Session)
   */
  protected NodeRecord findNodeByPath(String path, Session hibernateSession) {
    
    MultiRDBNodeRecord r = (MultiRDBNodeRecord) hibernateSession.createQuery(FIND_NODE_BY_PATH)
      .setString(0, path).uniqueResult();
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findNodeByUUID(java.lang.String, org.hibernate.Session)
   */
  protected NodeRecord findNodeByUUID(String uuid, Session hibernateSession) {
    
    MultiRDBNodeRecord r = (MultiRDBNodeRecord) hibernateSession.get(MultiRDBNodeRecord.class,
        uuid);
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findPropertyByPath(java.lang.String, org.hibernate.Session)
   */
  protected PropertyRecord findPropertyByPath(String path,
      Session hibernateSession) {
    
    MultiRDBPropertyRecord r = (MultiRDBPropertyRecord) hibernateSession.createQuery(
        FIND_PROPERTY_BY_PATH).setString(0, path).uniqueResult();
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findPropertyByUUID(java.lang.String, org.hibernate.Session)
   */
  protected PropertyRecord findPropertyByUUID(String uuid,
      Session hibernateSession) {
    
    MultiRDBPropertyRecord r = (MultiRDBPropertyRecord) hibernateSession.get(
        MultiRDBPropertyRecord.class, uuid);
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findReferences(java.lang.String, org.hibernate.Session)
   */
  protected List findReferences(String uuid, Session hibernateSession) {
    // [PN] try use first commented block (FIND_PROPS_BY_TYPE_AND_VALUE) 
    // instead current (FIND_PROPS_BY_VALUE)
    
    //  return hibernateSession.createQuery(FIND_PROPS_BY_TYPE_AND_VALUE)
    //    .setInteger(0, PropertyType.REFERENCE).setBinary(1, uuid.getBytes())
    //    .list();
    
    List references = hibernateSession.createQuery(FIND_PROPS_BY_VALUE).setBinary(0,
        uuid.getBytes()).list();
    /* // [PN] 23.03.06 Prev impl.
    List props = new ArrayList();
    for (int i = 0; i < references.size(); i++) {
      ValueRecord rec = (ValueRecord) references.get(i);
      if (rec.getProperty().getType() == PropertyType.REFERENCE) {
        props.add(rec.getProperty());
      }
    }
     */
    return references;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#getChildNodesByParentUUID(java.lang.String, org.hibernate.Session)
   */
  protected List getChildNodesByParentUUID(String parentUUID, Session hibernateSession) {
    List childNodes = hibernateSession.createQuery(FIND_NODES_BY_PARENTUUID).setString(0, 
        parentUUID).list();
    return childNodes;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#getChildPropertiesByParentUUID(java.lang.String, org.hibernate.Session)
   */
  protected List getChildPropertiesByParentUUID(String parentUUID, Session hibernateSession) {
    List childProperties = hibernateSession.createQuery(FIND_PROPERTIES_BY_PARENTUUID).setString(0, 
        parentUUID).list();
    return childProperties;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateWorkspaceDataContainer#getChildNodesCountByParentUUID(java.lang.String, org.hibernate.Session)
   */
  @Override
  protected int getChildNodesCountByParentUUID(String parentUUID, Session hibernateSession) {
    Integer r = (Integer) hibernateSession.createQuery(FIND_NODESCOUNT_BY_PARENTUUID).setString(0, 
        parentUUID).uniqueResult();
    return r.intValue();
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#getChildPropertiesCountByParentUUID(java.lang.String, org.hibernate.Session)
   */
  @Override
  protected int getChildPropertiesCountByParentUUID(String parentUUID, Session hibernateSession) {
    Integer r = (Integer) hibernateSession.createQuery(FIND_PROPERTIESCOUNT_BY_PARENTUUID).setString(0, 
        parentUUID).uniqueResult();
    return r.intValue();
  }  
  
  
  // experimental
  protected void initIndexes() {
    // JCR_MITEM table
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MITEM_ID ON JCR_MITEM(ID)");
//    } catch(Exception e) {
//      log.error("Error init JCR_IDX_MITEM_ID index " + e.getMessage());
//    }
   
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MITEM_PATH ON JCR_MITEM(PATH)");
//    } catch(Exception e) {
//      log.error("Error init JCR_IDX_MITEM_PATH index " + e.getMessage());
//    }
    
//    // JCR_MNODE table
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MNODE_ID ON JCR_MNODE(ID)");
//    } catch(Exception e) {
//      log.error("Error init JCR_IDX_MNODE_ID index " + e.getMessage());
//    }
//    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MNODE_PID ON JCR_MNODE(PARENT_ID)");
//    } catch(Exception e) {
//      log.error("Error init JCR_IDX_MNODE_PID index " + e.getMessage());
//    }
//
//    // JCR_MPROPERTY table    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MPROPERTY_ID ON JCR_MPROPERTY(ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_MPROPERTY_ID index " + e.getMessage());
//    }
//    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MPROPERTY_PID ON JCR_MPROPERTY(PARENT_ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_MPROPERTY_PID index " + e.getMessage());
//    }
//
//    // JCR_MPROPERTY table    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MVALUE_ID ON JCR_MVALUE(ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_MVALUE_PID index " + e.getMessage());
//    }
//    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_MVALUE_PID ON JCR_MVALUE(PROPERTY_ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_MVALUE_PID index " + e.getMessage());
//    }
  }
}
