/****************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.rdb.container.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedPropertyData;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedValueData;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.ValueData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: MultiRDBPropertyRecord.java 5623 2006-05-19 16:13:09Z peterit $
 * @hibernate.joined-subclass table="JCR_MPROPERTY" dynamic-update="true" l--azy="true"
 * @hibernate.joined-subclass-key column="ID"
 * @hi--bernate.cache usage="read-write"
 */

public class MultiRDBPropertyRecord extends MultiRDBItemRecord implements PropertyRecord {

  public void increasePersistedVersion() {
    throw new UnsupportedOperationException("Method not supported");
  }

  protected Set     propertyValues;

  protected Integer valuesType;

  protected Boolean multiValued = new Boolean(false);

  public MultiRDBPropertyRecord() {
  }

//  /**
//   * @h--ibernate.many-to-one column="ID"
//   *    class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBItemRecord"
//   *    foreign-key="JCR_FK_PROPERTY_ITEM"
//   */
//  public String getId() {
//    return super.getId();
//  }
//
//  public void setId(String id) {
//    super.setId(id);
//  }  
  
  /**
   * ---lazy="proxy"---
   * 
   * @hibernate.set cascade="all" inverse="true" l--azy="true"
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBValueRecord"
   * @hibernate.collection-key column="PROPERTY_ID" not-null="true"
   * @hibe---rnate.collection-index column="ORDER_NUM"
   */
  public Set getPropertyValues() {

    return propertyValues;
  }

  public void setPropertyValues(Set values) {

    this.propertyValues = values;
  }

  /**
   * @hibernate.property column="TYPE" not-null="true"
   */
  public Integer getValuesType() {
    //System.err.println("==== getValuesType(): " + valuesType.intValue());
    return valuesType;
  }

  public void setValuesType(Integer type) {
    this.valuesType = type;

  }

  /**
   * @hibernate.many-to-one column="PARENT_ID" not-null="true"
   *                        class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord"
   *                        foreign-key="JCR_FK_PROPERTY_NODE"
   */
  public NodeRecord getParentNode() {
    //toLog("==== PROP getParentNode(): " + (parentNode != null ? parentNode.getUUID() + "  " + parentNode.getPath() : "[null]") );
    return parentNode;
  }

  public void setParentNode(NodeRecord parent) {
    this.parentNode = parent;
    this.parentUUID = new String(parent.getId());
  }

  /**
   * @hibernate.property column="MULTIVALUED" not-null="true"
   */
  public Boolean isPropertyMultiValued() {
    return multiValued;
  }

  public void setPropertyMultiValued(Boolean multiValued) {
    this.multiValued = multiValued;
  }

  // ------------------ PropertyData --------------------------

  public int getType() {
    return getValuesType().intValue();
  }

  public List getValues() {

    ArrayList valueList = null;
    if (propertyValues != null) {
      valueList = new ArrayList(propertyValues);
    }
    return valueList;
  }

  public void setType(int type) {
    this.setValuesType(new Integer(type));
  }

  // Value Datas
  public void setValues(List values) throws RepositoryException {
    if (values == null)
      throw new RepositoryException("Set values failed for '" + path + "' values == null");
    
    propertyValues = new HashSet();
    // setPropertyValues(propertyValues);
    for (int i = 0; i < values.size(); i++) {
      MultiRDBValueRecord val = new MultiRDBValueRecord();
      val.setData(((ValueData) values.get(i)).getAsByteArray());
      val.setProperty(this);
      val.setOrderNumber(i);
      String s = val.getProperty().getPath();
      propertyValues.add(val);
    }
  }

  public boolean isNode() {
    return false;
  }

  public boolean isMultiValued() {
    return isPropertyMultiValued().booleanValue();
  }

  public void setMultiValued(boolean multivalued) {
    setPropertyMultiValued(new Boolean(multivalued));
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord#makeCopy()
   */
  public ItemData makeCopy() throws RepositoryException {
    List vs = new ArrayList();  
    int i=0;
    for (Iterator values = this.getPropertyValues().iterator(); values.hasNext();i++) {
      byte[] data = ((ValueRecord)values.next()).getAsByteArray();
      vs.add(new PersistedValueData(data, i));
    }
    return new PersistedPropertyData(id, InternalQPath.parse(path), parentNode.getId(), version,
        valuesType, multiValued, vs,  parentNode.getChildrenACL());
//    return null;
  }


  // ---------------------------------------------------------

  public String toString() {
    return "PropertyRecord: " + path + " type: "
        + ExtendedPropertyType.nameFromValue(this.getType()) + " uuid: " + getUUID() + " values: "
        + getPropertyValues();
  }

}