/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.value;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.SimpleParameterEntry;
import org.exoplatform.services.jcr.config.ValueStorageEntry;
import org.exoplatform.services.jcr.config.ValueStorageFilterEntry;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.storage.value.ValueIOChannel;
import org.exoplatform.services.jcr.storage.value.ValuePluginFilter;
import org.exoplatform.services.jcr.storage.value.ValueStoragePlugin;
import org.exoplatform.services.log.ExoLogger;
/**
 * Created by The eXo Platform SARL        .
 * Per-workspace factory object for ValueStoragePlugin
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */

public class ValueStoragePluginProvider extends ArrayList <ValueStoragePlugin> {
  
  private static Log log = ExoLogger.getLogger("jcr.ValueStoragePluginFactory");
  
  public static ValueStoragePluginProvider getInstance(WorkspaceEntry wsConfig) 
  throws RepositoryException, RepositoryConfigurationException, IOException {
//    log.info("Value Storage Plugin >>>>>>>>>>>>>>>>>>>> "+wsConfig.getContainer().getValueStorages());

    if(wsConfig.getContainer().getValueStorages() != null)
      return new ValueStoragePluginProvider(wsConfig);
    else 
      return null;
  }  
  
  public ValueStoragePluginProvider(WorkspaceEntry wsConfig/*, LocationFactory locationFactory*/) 
  throws RepositoryException, RepositoryConfigurationException, IOException {
    

    List <ValueStorageEntry> storages = wsConfig.getContainer().getValueStorages();
    
    //log.info("Value Storage Plugin >>>>>>>>>>>>>>>>>>>> "+storages);

    for(ValueStorageEntry storageEntry:storages) {
      Object o = null;
      try {
        o = Class.forName(storageEntry.getType()).newInstance();
      } catch (Exception e) {
        log.error("Value Storage Plugin instantiation FAILED. ", e);
        //e.printStackTrace();
        continue;
      } 
      if(!(o instanceof ValueStoragePlugin)) {
        log.error("Not a ValueStoragePlugin object IGNORED: "+o);
        continue;
      }
      ValueStoragePlugin plugin = (ValueStoragePlugin) o;
      // init filters
      ArrayList <ValuePluginFilter> filters = new ArrayList <ValuePluginFilter>();
      List <ValueStorageFilterEntry> filterEntries = storageEntry.getFilters();
      for(ValueStorageFilterEntry filterEntry : filterEntries) {
        
        // TODO
        ValuePluginFilter filter = new ValuePluginFilter(
            PropertyType.valueFromName(filterEntry.getPropertyType()),
            null,
            null);
//            locationFactory.parseAbsPath(filterEntry.getAncestorPath()).getInternalPath(), 
//            locationFactory.parseJCRName(filterEntry.getPropertyName()).getInternalName());
        filters.add(filter);
      }
      
      // init properties
      Properties props = new Properties();
      List <SimpleParameterEntry> paramEntries = storageEntry.getParameters();
      for(SimpleParameterEntry paramEntry : paramEntries) {
        props.setProperty(paramEntry.getName(), paramEntry.getValue());
      }
      plugin.init(props, filters);
      add(plugin);
      log.info("Value Storage Plugin initialized "+plugin);
    }
  }
  
  /**
   * @param property
   * @return ValueIOChannel appropriate for this property (by path, id etc)
   * or null if no such channel found 
   * @throws IOException
   */
  public ValueIOChannel getApplicableChannel(PropertyData property) throws IOException {
    Iterator plugins = iterator();
    while(plugins.hasNext()) {
      ValueStoragePlugin plugin = (ValueStoragePlugin)plugins.next();
      List <ValuePluginFilter> filters = plugin.getFilters();
      for(ValuePluginFilter filter:filters) {
        if(filter.match(property))
          return plugin.openIOChannel();
      }
    }
    return null;
  }
  
}
