/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.value.fs;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.jcr.dataflow.persistent.PersistedValueData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.storage.value.ValueIOChannel;
/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */

public class FileIOChannel implements ValueIOChannel {
  
  //private final int DEFAULT_BUFFER_SIZE = 2048;
  
  private File rootDir;
  
  public FileIOChannel(File rootDir) {
    this.rootDir = rootDir;
  }

  public List <ValueData> read(String propertyId) throws IOException {
    File[] valueFiles = rootDir.listFiles(new PropertyIDFilter(propertyId));
    int i=0;
    ArrayList <ValueData> values = new ArrayList <ValueData>(valueFiles.length); 
    for(File valueFile: valueFiles) {
      try {
        FileInputStream fis = new FileInputStream(valueFile);
        FileChannel channel = fis.getChannel();
        int max = Integer.MAX_VALUE;
        if (channel.size() > max) {
          throw new IOException("File too large: " + valueFile.getPath());
        }
        int size = (int) channel.size();
        ByteBuffer buf = ByteBuffer.allocate(size);
        int numRead = channel.read(buf);

        byte[] arr = new byte[numRead]; // buf.position()
        buf.rewind();
        buf.get(arr);
        values.add(new PersistedValueData(arr, i++));
        
        channel.close();
//        System.out.println("READ after close: "+valueFile.exists());

      } catch (FileNotFoundException e) {
        // never happens
      }
    }
    return values;
  }

  public void write(String propertyId, List <ValueData> values)  throws IOException {
//    File[] valueFiles = rootDir.listFiles(new PropertyIDFilter(propertyId));
//    // remove old files
//    for(File valueFile: valueFiles) {
//      valueFile.delete();
//    }

    // and write new
    for(ValueData value: values) {
      try {
        String fileName = propertyId+value.getOrderNumber();
        File file = new File(rootDir, fileName);
        FileOutputStream fos = new FileOutputStream(file);
//        ByteBuffer buf = ByteBuffer.allocate(DEFAULT_BUFFER_SIZE);
        ByteBuffer buf = ByteBuffer.wrap(value.getAsByteArray());
        FileChannel channel = fos.getChannel();
//        System.out.println("WRITE: "+propertyId+" "+value.getOrderNumber());
        channel.write(buf);
        channel.close();
        //fos.flush();
        fos.close();

      } catch (FileNotFoundException e) {
        // never happens
      }
    }
  }
  
  public void delete(String propertyId)  throws IOException {
    File[] valueFiles = rootDir.listFiles(new PropertyIDFilter(propertyId));
    // remove old files
    for(File valueFile: valueFiles) {
      boolean res = valueFile.delete();
    }
  }

  public void close() {
  }
  
  private class PropertyIDFilter implements FileFilter {
    
    private String id;

    public PropertyIDFilter(String id) {
      this.id = id;
    }

    public boolean accept(File filename) {
      return filename.getName().startsWith(id);
    }
  }
}
