/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.api.xa;


import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.core.XASession;
import org.exoplatform.services.transaction.TransactionService;

/**
 * Created by The eXo Platform SARL        . <br>
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: $
 */
public class TestUserTransaction extends JcrAPIBaseTest{

  private TransactionService ts;

  public void setUp() throws Exception {
    
//    StandaloneContainer.setConfigurationPath("src/java/conf/standalone/test-configuration.xml");
//    
//    container = StandaloneContainer.getInstance();
    
    super.setUp();
    
    ts = (TransactionService)container.
    getComponentInstanceOfType(TransactionService.class);
    
  }

  public void testCommit() throws Exception {
    
    UserTransaction ut = ts.getUserTransaction();
    ut.begin();
    session.getRootNode().addNode("txcommit");
    session.save();
    assertNotNull(session.getItem("/txcommit"));
    Session s1 = repository.login(new SimpleCredentials("admin","admin".toCharArray()), 
        session.getWorkspace().getName());
    try {
      assertNotNull(s1.getItem("/txcommit"));
      fail("PathNotFoundException should have be thrown");
    } catch (PathNotFoundException e) {
    } 
    ut.commit();
    assertNotNull(s1.getItem("/txcommit"));
  }

  public void testRollback() throws Exception {
    
    UserTransaction ut = ts.getUserTransaction();
    
    ut.begin();
    session.getRootNode().addNode("txrollback");
    session.save();
    assertNotNull(session.getItem("/txrollback"));
    
//    Session s1 = repository.login(new SimpleCredentials("admin","admin".toCharArray()), 
//        session.getWorkspace().getName());
    ut.rollback();
    try {
      assertNotNull(session.getItem("/txrollback"));
      fail("PathNotFoundException should have be thrown");
    } catch (PathNotFoundException e) {
    } 

  }
  public void testUserTransactionFromJndi() throws Exception {

    InitialContext ctx = new InitialContext();
    Object obj = ctx.lookup("UserTransaction");
    UserTransaction ut = (UserTransaction) obj;

    ut.begin();
    Session s1 = repository.login(new SimpleCredentials("admin","admin".toCharArray()), 
     session.getWorkspace().getName());
    s1.getRootNode().addNode("txcommit1");
    s1.save();
    ut.commit();
    assertNotNull(session.getItem("/txcommit1"));

  }
  
  public void testReuseUT() throws Exception {

    InitialContext ctx = new InitialContext();
    Object obj = ctx.lookup("UserTransaction");
    UserTransaction ut = (UserTransaction) obj;
    
    Session s1 = repository.login(new SimpleCredentials("admin","admin".toCharArray()), 
        session.getWorkspace().getName());

    
    ut.begin();
    Node tx2 = s1.getRootNode().addNode("txcommit2");
    ut.commit();
    
    // In a case of reusing Have to enlist the resource once again!
    ((XASession)s1).enlistResource();
    
    ut.begin();
    tx2.addNode("txcommit21");
    s1.save();
    ut.commit();
    assertNotNull(session.getItem("/txcommit2/txcommit21"));
    
  }

}
