package org.exoplatform.services.webdav;

import static org.apache.jackrabbit.server.AbstractWebdavServlet.DEFAULT_AUTHENTICATE_HEADER;

import java.io.IOException;
import java.net.URL;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.chain.config.ConfigParser;
import org.apache.jackrabbit.server.AbstractWebdavServlet;
import org.apache.jackrabbit.webdav.DavException;
import org.apache.jackrabbit.webdav.DavLocatorFactory;
import org.apache.jackrabbit.webdav.DavMethods;
import org.apache.jackrabbit.webdav.DavResource;
import org.apache.jackrabbit.webdav.DavResourceFactory;
import org.apache.jackrabbit.webdav.DavServletResponse;
import org.apache.jackrabbit.webdav.DavSessionProvider;
import org.apache.jackrabbit.webdav.WebdavRequest;
import org.apache.jackrabbit.webdav.WebdavResponse;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;

/**
 * @deprecated
 */
public class ExoWebdavServlet extends AbstractWebdavServlet {

  /** init param name of the repository prefix */
  public static final String INIT_PARAM_RESOURCE_PATH_PREFIX = "resource-path-prefix";

  /** init param file of the commons chain catalog */
  public static final String INIT_PARAM_CHAIN_CATALOG = "chain-catalog";  
  
  private static final String INIT_PARAM_AUTHENTICATE_HEADER = "authenticate-header";  
  
  /** the repository prefix retrieved from config */
  private static String resourcePathPrefix;
  
  private static String authenticate_header;

  /**
   * Init this servlet
   * 
   * @throws ServletException
   */
  public void init() throws ServletException {
    super.init();
    resourcePathPrefix = getInitParameter(INIT_PARAM_RESOURCE_PATH_PREFIX);    
    if (resourcePathPrefix == null) {
      resourcePathPrefix = "";
    } else if (resourcePathPrefix.endsWith("/")) {
      resourcePathPrefix = resourcePathPrefix.substring(0, resourcePathPrefix
          .length() - 1);
    }
/*
    try {
      String chain = getInitParameter(INIT_PARAM_CHAIN_CATALOG);
      URL chainUrl = getServletContext().getResource(chain);
      ConfigParser parser = new ConfigParser();
      parser.parse(chainUrl);
    } catch (Exception e) {
      throw new ServletException(e);
    }
*/    
    authenticate_header = getInitParameter(INIT_PARAM_AUTHENTICATE_HEADER);
    if (authenticate_header == null) {
        authenticate_header = DEFAULT_AUTHENTICATE_HEADER;
    }
  }
  
//  public void service(HttpServletRequest request, HttpServletResponse response) 
//  throws ServletException, IOException {
//    System.out.println("DAV REQUEST "+request.getRequestURI());
//    super.service(request, response);
//  }
  
  /**
   * Executes the respective method in the given webdav context.
   * The method is overridden since not all webdav methods should be
   * supported by this servlet.
   *
   * @param request
   * @param response
   * @param method
   * @param resource
   * @return
   * @throws ServletException
   * @throws IOException
   * @throws DavException
   */
  protected boolean execute(WebdavRequest request, WebdavResponse response,
                            int method, DavResource resource)
          throws ServletException, IOException, DavException {
      /* set cache control headers in order to deal with non-dav complient
      * http1.1 or http1.0 proxies. >> see RFC2518 9.4.5 */
      response.addHeader("Pragma", "No-cache");  // http1.0
      response.addHeader("Cache-Control", "no-cache"); // http1.1

      switch (method) {
          case DavMethods.DAV_HEAD:
              doHead(request, response, resource);
              break;
          case DavMethods.DAV_GET:
              doGet(request, response, resource);
              break;
          case DavMethods.DAV_OPTIONS:
              doOptions(request, response, resource);
              break;
          case DavMethods.DAV_PROPFIND:
              doPropFind(request, response, resource);
              break;
          case DavMethods.DAV_PROPPATCH:
              doPropPatch(request, response, resource);
              break;
          case DavMethods.DAV_PUT:
              doPut(request, response, resource);
              break;
          case DavMethods.DAV_POST:
              doPost(request, response, resource);
              break;
          case DavMethods.DAV_DELETE:
              doDelete(request, response, resource);
              break;
          case DavMethods.DAV_COPY:
              doCopy(request, response, resource);
              break;
          case DavMethods.DAV_MOVE:
              doMove(request, response, resource);
              break;
          case DavMethods.DAV_MKCOL:
              doMkCol(request, response, resource);
              break;
          case DavMethods.DAV_LOCK:
              doLock(request, response, resource);
              break;
          case DavMethods.DAV_UNLOCK:
              doUnlock(request, response, resource);
              break;
          default:
              // any other method
              return false;
      }      
      return true;
  }

  /**
   * The MKCOL method
   *
   * @throws IOException
   */
  protected void doMkCol(WebdavRequest request, WebdavResponse response,
                         DavResource resource) throws IOException, DavException {
      // mkcol request with request.body is not supported.
      if (request.getContentLength() > 0 || request.getHeader("Transfer-Encoding") != null) {
          response.sendError(DavServletResponse.SC_UNSUPPORTED_MEDIA_TYPE);
          return;
      }
      super.doMkCol(request, response, resource);
  }
  

  protected boolean isPreconditionValid(WebdavRequest request, DavResource resource) {
    return !resource.exists() || request.matchesIfHeader(resource);
  }


  public DavSessionProvider getDavSessionProvider() {
    //PortalContainer pcontainer = PortalContainer.getInstance();
    ExoContainer container = ExoContainerContext.getCurrentContainer();
    WebdavService webdavService = (WebdavService) container.getComponentInstanceOfType(
        WebdavService.class);
    System.out.println(">>>>>>>>getDavSessionProvider>>>>>>>>"+resourcePathPrefix+" "+webdavService+" "+container);

    return webdavService.getDavSessionProvider();
  }


  public void setDavSessionProvider(DavSessionProvider davSessionProvider) {}


  public DavLocatorFactory getLocatorFactory() {
    //PortalContainer pcontainer = PortalContainer.getInstance();
    ExoContainer container = ExoContainerContext.getCurrentContainer();
    WebdavService webdavService = (WebdavService) container.getComponentInstanceOfType(
        WebdavService.class);   
    System.out.println(">>>>>>>>getLocatorFactory>>>>>>>>"+resourcePathPrefix+" "+webdavService+" "+container);
    return webdavService.getLocatorFactory(resourcePathPrefix);
  }

  public void setLocatorFactory(DavLocatorFactory davLocatorFactory) {}


  public DavResourceFactory getResourceFactory() {
    ExoContainer container = ExoContainerContext.getCurrentContainer();
    //PortalContainer pcontainer = PortalContainer.getInstance();
    WebdavService webdavService = (WebdavService) container.getComponentInstanceOfType(
        WebdavService.class);    
    System.out.println(">>>>>>>>getResourceFactory>>>>>>>>"+resourcePathPrefix+" "+webdavService+" "+container);
    return webdavService.getResourceFactory();    
  }

  public void setResourceFactory(DavResourceFactory resourceFactory) {}


  public String getAuthenticateHeaderValue() {    
    return authenticate_header;
  }
  
}
