package org.exoplatform.services.webdav.impl;

import org.apache.jackrabbit.webdav.DavLocatorFactory;
import org.apache.jackrabbit.webdav.DavResourceFactory;
import org.apache.jackrabbit.webdav.DavSessionProvider;
import org.apache.jackrabbit.webdav.lock.LockManager;
import org.apache.jackrabbit.webdav.lock.SimpleLockManager;
import org.apache.jackrabbit.webdav.simple.LocatorFactoryImpl;
import org.apache.jackrabbit.webdav.simple.ResourceFactoryImpl;
import org.exoplatform.services.webdav.WebdavService;

public class WebdavServiceImpl implements WebdavService {
  
  /**
   * Map used to remember any webdav lock created without being reflected in the
   * underlaying repository. This is needed because some clients rely on a
   * successful locking mechanism in order to perform properly (e.g. mac OSX
   * built-in dav client)
   */
  private LockManager lockManager;

  /** the resource factory */
  private DavResourceFactory resourceFactory;

  /** the locator factory */
  private DavLocatorFactory locatorFactory;

  /** the session provider */
  private DavSessionProvider sessionProvider;

  public WebdavServiceImpl(DavSessionProvider sessionProvider) throws Exception {
    //System.out.println(">>>>>>>>>>>>>>>> --------- "+this+" "+sessionProvider);
    this.sessionProvider = sessionProvider;
  }
  
  public DavLocatorFactory getLocatorFactory(String resourcePathPrefix) {
    if (locatorFactory == null) {
      locatorFactory = new LocatorFactoryImpl(resourcePathPrefix);
    }
    return locatorFactory;
  }

  public LockManager getLockManager() {
    if (lockManager == null) {
      lockManager = new SimpleLockManager();
    }
    return lockManager;
  }

  public DavResourceFactory getResourceFactory() {
    if (resourceFactory == null) {
      resourceFactory = new ResourceFactoryImpl(getLockManager());
    }
    return resourceFactory;
  }

  public DavSessionProvider getDavSessionProvider() {
    return sessionProvider;
  }

  public void setLocatorFactory(DavLocatorFactory locatorFactory) {
    this.locatorFactory = locatorFactory;
  }

  public void setLockManager(LockManager lockManager) {
    this.lockManager = lockManager;
  }

  public void setResourceFactory(DavResourceFactory resourceFactory) {
    this.resourceFactory = resourceFactory;
  }

  public void setSessionProvider(DavSessionProvider sessionProvider) {
    this.sessionProvider = sessionProvider;
  }
  
}

